import json

from jose import jwt


class AuthenticationTokenException(Exception):
    def __init__(self, error):
        super().__init__(error)


def extract_claims(token: str) -> list:
    if not token:
        raise AuthenticationTokenException(
            "Authentication error: Authorization header is missing"
        )

    parts: list[str] = token.split()

    if parts[0].lower() != "bearer":
        raise AuthenticationTokenException(
            "Authentication error: Authorization header must start with ' Bearer'"
        )
    elif len(parts) == 1:
        raise AuthenticationTokenException("Authentication error: Token not found")
    elif len(parts) > 2:
        raise AuthenticationTokenException(
            "Authentication error: Authorization header must be 'Bearer <token>'"
        )

    token_claims: dict[str, any] = jwt.get_unverified_claims(parts[1])
    return token_claims


def validate_claims(
    token_claims: dict[str, any], required_claims: dict[str, any]
) -> bool:
    for key in required_claims:
        if key not in token_claims or token_claims[key] != required_claims[key]:
            raise AuthenticationTokenException(
                "Authentication error: The token provided does not have sufficiently valid claims to authorize access to this service."
            )

    return True



import re
class APIKeyException(Exception):
    pass

def validate_email(email):
    # Define a regular expression for email validation
    email_regex = r"^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$"

    # Return True if the email matches the regex, False otherwise
    return re.match(email_regex, email) is not None


def validate_scopes(user_scopes, acceptable_scopes):
    has_scope = False
    for user_scope in user_scopes:
        if user_scope == "godmode":
            has_scope = True
            break 
        
        if user_scope in acceptable_scopes:
            has_scope = True
            break

    if not has_scope:
        raise APIKeyException("User does not have appropriate access scopes for this endpoint")

def validate_required_parameters(body, required_arguments):
    missing_values = [value for value in required_arguments if value not in body]
    if missing_values:
        raise ValueError("Missing or empty parameter in request body. Requires: {}".format(", ".join(missing_values)))

def get_payload(req):
    if type(req) == dict:
        return req
    
    return req.get_json()


import urllib.request
import json
import logging


class AzureDeployedModel:
    """Wrapper class for Azure deployed model."""

    def __init__(
        self,
        endpoint_url: str,
        api_key: str,
    ) -> None:
        self.endpoint_url = endpoint_url
        self.api_key = api_key

    def _convert_bytes_to_list(self, val: bytes) -> list:
        """Convert the bytes to list."""
        try:
            # Attempt to decode the input string as JSON
            result_dict = json.loads(val)

            if isinstance(result_dict, list):
                # Check if the parsed JSON is a list
                if result_dict:
                    return result_dict[0]
                else:
                    return []
            else:
                return []
        except json.JSONDecodeError:
            # Handle JSON decoding error
            return []


    def predict(self, input_blob: dict) -> list:
        """Invoke the deployed model and return the result."""
        body = str.encode(json.dumps(input_blob))

        if not self.api_key:
            raise Exception("A key should be provided to invoke the endpoint")

        headers = {
            "Content-Type": "application/json",
            "Authorization": ("Bearer " + self.api_key),
            "azureml-model-deployment": "high-level-categorize-queries-2",
        }

        req = urllib.request.Request(self.endpoint_url, body, headers)

        try:
            response = urllib.request.urlopen(req)

            result = response.read()
            return self._convert_bytes_to_list(result)

        except urllib.error.HTTPError as error:
            logging.info("The request failed with status code: " + str(error.code))

            logging.info(error.info())
            logging.info(error.read().decode("utf8", "ignore"))



from kbrainsdk.validation.users import validate_user_entra_groups, validate_list_group_members
from kbrainsdk.apibase import APIBase

class User(APIBase):
    
    def user_entra_groups(self, email, token, client_id, oauth_secret, tenant_id):    
            payload = {
                "email": email,
                "token": token,
                "client_id": client_id,
                "oauth_secret": oauth_secret,
                "tenant_id": tenant_id
            }
    
            validate_user_entra_groups(payload)
    
            path = f"/user/entra/groups/v1"
            response = self.apiobject.call_endpoint(path, payload, "post")
            return response

    
    def list_group_members(self, group_id, email, token, client_id, oauth_secret, tenant_id, continuation_token=None):    
            payload = {
                "group_id": group_id,
                "email": email,
                "token": token,
                "client_id": client_id,
                "oauth_secret": oauth_secret,
                "tenant_id": tenant_id,
            }
    
            validate_list_group_members(payload)
    
            if continuation_token:
                payload["continuation_token"] = continuation_token
                
            path = f"/user/list/groups/list/v1"
            response = self.apiobject.call_endpoint(path, payload, "post")
            return response

            


from kbrainsdk.validation.common import get_payload, validate_email, validate_required_parameters
def validate_ingest_onedrive(req):
    body = get_payload(req)
    email = body.get('email')
    token = body.get('token')
    environment = body.get('environment')
    client_id = body.get('client_id')
    oauth_secret = body.get('oauth_secret')
    tenant_id = body.get('tenant_id')

    required_arguments = ["email", "token", "environment", "client_id", "oauth_secret", "tenant_id"]
    validate_required_parameters(body, required_arguments)

    if not validate_email(email):
        raise ValueError("Invalid email address")
    
    return email, token, environment, client_id, oauth_secret, tenant_id

def validate_ingest_sharepoint_scan(req):
    body = get_payload(req)
    host = body.get('host')
    site = body.get('site')
    token = body.get('token')
    environment = body.get('environment')
    client_id = body.get('client_id')
    oauth_secret = body.get('oauth_secret')
    tenant_id = body.get('tenant_id')

    required_arguments = ["host", "site", "token", "environment", "client_id", "oauth_secret", "tenant_id"]
    validate_required_parameters(body, required_arguments)

    drive = body.get('drive', "")
    folder_id = body.get('folder_id', "")
    current_next_link = body.get('current_next_link', "")
    initial_run_id = body.get('initial_run_id', "")

    return host, site, token, environment, client_id, oauth_secret, tenant_id, folder_id, drive, current_next_link, initial_run_id

def validate_ingest_sharepoint_files(req):
    body = get_payload(req)
    host = body.get('host')
    site = body.get('site')
    token = body.get('token')
    environment = body.get('environment')
    client_id = body.get('client_id')
    oauth_secret = body.get('oauth_secret')
    tenant_id = body.get('tenant_id')
    drive = body.get('drive', "")
    initial_run_id = body.get('initial_run_id', "")
    cosmos_guid = body.get('cosmos_guid', "")
    required_arguments = ["host", "site", "token", "environment", "client_id", "oauth_secret", "tenant_id", "drive", "initial_run_id", "cosmos_guid"]
    validate_required_parameters(body, required_arguments)

    folder_id = body.get('folder_id', "")
    current_next_link = body.get('current_next_link', "")
    
    return host, site, token, environment, client_id, oauth_secret, tenant_id, folder_id, drive, current_next_link, initial_run_id, cosmos_guid


def validate_ingest_status(req):
    body = get_payload(req)
    datasource = body.get('datasource')

    required_arguments = ["datasource"]
    validate_required_parameters(body, required_arguments)
    
    return datasource

def validate_ingest_pipeline_status(req):
    body = get_payload(req)
    run_id = body.get('run_id')

    required_arguments = ["run_id"]
    validate_required_parameters(body, required_arguments)

    return run_id


from typing import Any
from kbrainsdk.validation.events import validate_servicebus_message
from kbrainsdk.apibase import APIBase

class Events(APIBase):

    def __init__(self, *args: Any, **kwds: Any) -> Any:
        return super().__init__(*args, **kwds)
    
    def publish_message(self, message: str, topic_name: str, application_properties: dict | None = None) -> None:
        payload = {
            "message": message,
            "topic_name": topic_name,
            "application_properties": application_properties
        }
        
        validate_servicebus_message(payload)
        path = f"/service_bus/send/v1"
        response = self.apiobject.call_endpoint(path, payload, "post")
        return response



from kbrainsdk.validation.common import get_payload, validate_email, validate_required_parameters


def validate_user_entra_groups(req):
    body = get_payload(req)
    required_arguments = ["email", "token", "client_id", "oauth_secret", "tenant_id"]
    validate_required_parameters(body, required_arguments)
    
    email = body.get('email')
    token = body.get('token')
    client_id = body.get('client_id')
    oauth_secret = body.get('oauth_secret')
    tenant_id = body.get('tenant_id')

    if not validate_email(email):
        raise ValueError("Invalid email address")
    
    return email, token, client_id, oauth_secret, tenant_id

def validate_list_group_members(req):
    body = get_payload(req)
    required_arguments = ["group_id", "email", "token", "client_id", "oauth_secret", "tenant_id"]
    validate_required_parameters(body, required_arguments)

    group_id = body.get('group_id')
    email = body.get('email')
    token = body.get('token')
    client_id = body.get('client_id')
    oauth_secret = body.get('oauth_secret')
    tenant_id = body.get('tenant_id')
    continuation_token = body.get('continuation_token', None)

    if not validate_email(email):
        raise ValueError("Invalid email address")
    
    return group_id, email, token, client_id, oauth_secret, tenant_id, continuation_token


from typing import Any, Dict
from kbrainsdk.apibase import APIBase
from kbrainsdk.validation.signalr import (
    validate_signalr_broadcast, 
    validate_signalr_group_broadcast,
    validate_signalr_subscription_request
)

class SignalR(APIBase):

    def __init__(self, *args: Any, **kwds: Any) -> Any:
        return super().__init__(*args, **kwds)
    
    def broadcast(self, 
        target:str, 
        action: Dict[str, Any], 
    ):
        payload = {
            "target": target,
            "action": action
        }
        validate_signalr_broadcast(payload)
        path = f"/websocket/broadcast/v1"
        response = self.apiobject.call_endpoint(path, payload, "post")
        return response
    
    def group_broadcast(self, 
        group_name:str, 
        action: Dict[str, Any], 
        client_id: str,
        tenant_id: str,
        client_secret: str
    ):
        payload = {
            "group_name": group_name,
            "action": action,
            "client_id": client_id,
            "tenant_id": tenant_id,
            "client_secret": client_secret
        }
        validate_signalr_group_broadcast(payload)
        path = f"/websocket/group/broadcast/v1"
        response = self.apiobject.call_endpoint(path, payload, "post")
        return response

    def subscribe_to_websocket_group(self, token: str, subscription_id:str, group_name: str, client_id: str, tenant_id: str, client_secret: str):
        payload = {
            "subscription_id": subscription_id,
            "token": token,
            "group_name": group_name,
            "client_id": client_id,
            "tenant_id": tenant_id,
            "client_secret": client_secret
        }
        validate_signalr_subscription_request(payload)
        path = f"/websocket/group/subscribe/v1"
        response = self.apiobject.call_endpoint(path, payload, "post")
        return response

    def unsubscribe_to_websocket_group(self, token: str, group_name: str, subscription_id: str, client_id: str, tenant_id: str, client_secret: str):
        payload = {
            "token": token,
            "group_name": group_name,
            "subscription_id": subscription_id,
            "client_id": client_id,
            "tenant_id": tenant_id,
            "client_secret": client_secret
        }
        validate_signalr_subscription_request(payload)
        path = f"/websocket/group/unsubscribe/v1"
        response = self.apiobject.call_endpoint(path, payload, "post")
        return response


import math
import re
import warnings
from kbrainsdk.validation.ingest import validate_ingest_onedrive, validate_ingest_sharepoint_scan, validate_ingest_sharepoint_files, validate_ingest_status, validate_ingest_pipeline_status
from kbrainsdk.apibase import APIBase
from docx import Document
import PyPDF2
import pandas as pd

class Ingest(APIBase):

    def __init__(self, *args, **kwds):
        self.FILE_HANDLER = {
            "pdf": self.pdf_handler,
            "txt": self.txt_handler,
            "docx": self.docx_handler
        }
        self.TXT_CHARACTERS_PER_PAGE = 2000  # Adjust this value based on your assumption
        self.HEADER_CUTOFF = 200
        self.DOCX_CHARACTERS_PER_PAGE = 500
        return super().__init__(*args, **kwds)

    def get_supported_file_types(self):
        return list(self.FILE_HANDLER.keys())

    def ingest_onedrive(self, email, token, client_id, oauth_secret, tenant_id, environment):

        payload = {
            "email": email,
            "token": token,
            "environment": environment,
            "client_id": client_id,
            "oauth_secret": oauth_secret,
            "tenant_id": tenant_id
        }

        validate_ingest_onedrive(payload)

        path = f"/ingest/onedrive/v1"
        response = self.apiobject.call_endpoint(path, payload, "post")
        return response

    def ingest_sharepoint_scan(self, host, site, token, client_id, oauth_secret, tenant_id, environment, folder_id=None, drive=None, current_next_link=None, initial_run_id=None):

        payload = {
            "host": host,
            "site": site,
            "token": token,
            "environment": environment,
            "client_id": client_id,
            "oauth_secret": oauth_secret,
            "tenant_id": tenant_id,
        }

        if folder_id:
            payload["folder_id"] = folder_id
        
        if drive:
            payload["drive"] = drive

        if current_next_link:
            payload["current_next_link"] = current_next_link

        if initial_run_id:
            payload["initial_run_id"] = initial_run_id
            
        validate_ingest_sharepoint_scan(payload)

        path = f"/ingest/sharepoint/scan/v1"
        response = self.apiobject.call_endpoint(path, payload, "post")
        return response
    
    def ingest_sharepoint_files(self, host, site, token, client_id, oauth_secret, tenant_id, environment, initial_run_id, drive, cosmos_guid, folder_id=None, current_next_link=None, ):

        payload = {
            "host": host,
            "site": site,
            "token": token,
            "environment": environment,
            "client_id": client_id,
            "oauth_secret": oauth_secret,
            "tenant_id": tenant_id,
            "drive": drive,
            "initial_run_id": initial_run_id,
            "cosmos_guid": cosmos_guid
        }

        if folder_id:
            payload["folder_id"] = folder_id
        
        if current_next_link:
            payload["current_next_link"] = current_next_link
            
        validate_ingest_sharepoint_files(payload)

        path = f"/ingest/sharepoint/files/v1"
        response = self.apiobject.call_endpoint(path, payload, "post")
        return response

    def get_status(self, datasource):

        payload = {
            "datasource": datasource 
        }

        validate_ingest_status(payload)

        path = f"/ingest/status/v1"
        response = self.apiobject.call_endpoint(path, payload, "post")
        return response
    
    def get_pipeline_status(self, run_id):

        payload = {
            "run_id": run_id 
        }

        validate_ingest_pipeline_status(payload)

        path = f"/ingest/pipeline/run/status/v1"
        response = self.apiobject.call_endpoint(path, payload, "post")
        return response

    def convert_site_to_datasource(self, site):
        # Remove special characters and replace spaces with hyphens
        site_name = re.sub(r'[^a-zA-Z0-9\s]', '', site)
        site_name = site_name.replace(' ', '-')
        return f"sharepoint-{site_name.lower().replace('.', '-')}"

    def convert_email_to_datasource(self, email):
        return f"drive-{email.lower().replace('@', '-at-').replace('.', '-')}"

    def pdf_handler(self, local_filepath, document_df, filename):
        with open(local_filepath, 'rb') as file:
            pdf_reader = PyPDF2.PdfReader(file)
            for page in range(len(pdf_reader.pages)):
                extracted_page = pdf_reader.pages[page]
                text = extracted_page.extract_text()
                sections = re.split(r'\t+|\n{2,}|\s{2,}', text)
                sections = list(filter(lambda x: x.strip() != '', sections))
                for text, section, section_type, paragraph in self.paragraph_generator(sections):
                    page_df = pd.DataFrame({
                        "text": [text],
                        "section": [section],
                        "file": [filename],
                        "page": [page + 1],
                        "type": [section_type],
                        "paragraph": [paragraph]
                    })
                    document_df = pd.concat([document_df, page_df], axis='rows')
        document_df.reset_index(drop=True, inplace=True)
        return document_df

    def estimate_pages(self, character_count, average_characters_per_page):
        return character_count // average_characters_per_page + 1


    def txt_handler(self, local_filepath, document_df, filename):
        with open(local_filepath, 'r') as file:
            text = file.read()
            character_count = len(text)
            estimated_pages = self.estimate_pages(character_count, self.TXT_CHARACTERS_PER_PAGE)
            paragraphs = re.split(r'\t+|\n{2,}|\s{2,}', text)
            paragraphs = list(filter(lambda x: x.strip() != '', paragraphs))
            total_sections = len(paragraphs)
            sections_per_page = math.ceil(total_sections / estimated_pages)
            for text, section, section_type, paragraph in self.paragraph_generator(paragraphs):
                page_df = pd.DataFrame({
                    "text": [text],
                    "section": [section],
                    "file": [filename],
                    "page": [int(math.ceil(section / sections_per_page))],
                    "type": [section_type],
                    "paragraph": [paragraph]
                })
                document_df = pd.concat([document_df, page_df], axis='rows')
        document_df.reset_index(drop=True, inplace=True)
        return document_df

    def docx_handler(self, local_filepath, document_df, filename):
        doc = Document(local_filepath)
        paragraphs = [p.text for p in doc.paragraphs]
        for text, section, section_type, paragraph in self.paragraph_generator(paragraphs):
            page_df = pd.DataFrame({
                "text": [text],
                "section": [section],
                "file": [filename],
                "page": [section // self.DOCX_CHARACTERS_PER_PAGE + 1],
                "type": [section_type],
                "paragraph": [paragraph]
            })
            document_df = pd.concat([document_df, page_df], axis='rows')
        document_df.reset_index(drop=True, inplace=True)
        return document_df
    
    def paragraph_generator(self, textblocks):
        previous_section = None
        paragraphs = 0
        sections = 0
        for text in textblocks:
            section_type = "header" if len(text) < self.HEADER_CUTOFF else "paragraph"
            if previous_section != "header":
                paragraphs += 1
            previous_section = section_type
            sections += 1
            yield text, sections, section_type, paragraphs
    ################################
    ##FLAWED AND DEPRECATED
    ################################
    def count_paragraphs(self, x, paragraph_count):
        warnings.warn(f"count_paragraphs contains bugs is deprecated. Use paragraph_generator instead to count paragraphs", DeprecationWarning, stacklevel=2)
        if x == "header":
            if paragraph_count["count"] == 0:
                return 1
            return paragraph_count["count"]
        
        paragraph_count["count"] += 1
        return int(paragraph_count["count"])

    def process_document(self, local_filepath, document_name, extension):
        try:
            document_df = pd.DataFrame({"file":[], "page":[], "section":[], "text":[], "type":[], "paragraph": []})
            filename = f"{document_name.strip()}.{extension}"
            document_df = self.FILE_HANDLER[extension.lower()](local_filepath, document_df, filename)
            metafile = f"{local_filepath.replace(f'.{extension}','')}_meta.csv"
            document_df.reset_index(inplace=True, drop=True)
            document_df.to_csv(metafile, escapechar="\\", index=False)                
            return metafile
        except Exception as ex:
            return str(ex)
            



from kbrainsdk.validation.common import get_payload, validate_required_parameters

def validate_servicebus_message(req):
    body = get_payload(req)
    required_arguments = ["message", "topic_name", "application_properties"]
    validate_required_parameters(body, required_arguments)

    message = body.get('message')
    topic_name = body.get('topic_name')
    application_properties = body.get('application_properties')    

    return message, topic_name, application_properties


from kbrainsdk.validation.common import get_payload, validate_required_parameters

def validate_signalr_broadcast(req):
    body = get_payload(req)
    required_arguments = ["target", "action"]
    validate_required_parameters(body, required_arguments)

    target = body.get('target')
    action = body.get('action')
    
    return target, action

def validate_signalr_group_broadcast(req):
    body = get_payload(req)
    required_arguments = ["group_name", "action", "client_id", "tenant_id", "client_secret"]
    validate_required_parameters(body, required_arguments)

    group_name = body.get('group_name')
    action = body.get('action')
    client_id = body.get('client_id')
    tenant_id = body.get('tenant_id')
    client_secret = body.get('client_secret')
    
    return group_name, action, client_id, tenant_id, client_secret

def validate_signalr_group_request(req):
    body = get_payload(req)
    required_arguments = ["token", "group_name", "client_id", "tenant_id", "client_secret"]
    validate_required_parameters(body, required_arguments)

    token = body.get('token')
    client_id = body.get('client_id')
    tenant_id = body.get('tenant_id')
    group_name = body.get('group_name')
    client_secret = body.get('client_secret')

    return token, group_name, client_id, tenant_id, client_secret

def validate_signalr_subscription_request(req):
    body = get_payload(req)
    required_arguments = ["token", "group_name", "subscription_id", "client_id", "tenant_id", "client_secret"]
    validate_required_parameters(body, required_arguments)

    token = body.get('token')
    client_id = body.get('client_id')
    tenant_id = body.get('tenant_id')
    group_name = body.get('group_name')
    subscription_id = body.get('subscription_id')
    client_secret = body.get('client_secret')

    return token, group_name, subscription_id, client_id, tenant_id, client_secret


from kbrainsdk.validation.datasets import validate_list_datasets, validate_search_datasets, validate_list_dataset_files, validate_get_focus_configuration, validate_update_focus_configuration, validate_list_focus_configurations
from kbrainsdk.apibase import APIBase

class Datasets(APIBase):

    def list_datasets(self, email, token, client_id, oauth_secret, tenant_id, selected_datasets = None, search_term = None, focus_chat_id = None):
        
        payload = {
            "email": email,
            "token": token,
            "client_id": client_id,
            "oauth_secret": oauth_secret,
            "tenant_id": tenant_id,
            "search_term": search_term,
            "selected_datasets": selected_datasets,
            "focus_chat_id": focus_chat_id
        }

        validate_list_datasets(payload)

        path = f"/datasets/list/v1"
        response = self.apiobject.call_endpoint(path, payload, "post")
        return response

    def list_files_in_datasets(self, email, token, client_id, oauth_secret, tenant_id, dataset_id, pagination = None, max_item_count=10, search_term = None):
        
        payload = {
            "email": email,
            "token": token,
            "client_id": client_id,
            "oauth_secret": oauth_secret,
            "tenant_id": tenant_id,
            "dataset_id": dataset_id,
            "pagination": pagination,
            "max_item_count": max_item_count,
            "search_term": search_term
        }

        validate_list_dataset_files(payload)

        path = f"/datasets/files/v1"
        response = self.apiobject.call_endpoint(path, payload, "post")
        return response

    def search_datasets(self, query, topic, citations, email, token, client_id, oauth_secret, tenant_id, selected_datasets = None, focus_chat_id = None):
        
        payload = {
            "email": email,
            "token": token,
            "client_id": client_id,
            "oauth_secret": oauth_secret,
            "tenant_id": tenant_id,
            "query": query,
            "topic": topic,
            "citations": citations,
            "focus_chat_id": focus_chat_id
        }

        if selected_datasets:
            payload["selected_datasets"] = selected_datasets

        validate_list_datasets(payload)
        validate_search_datasets(payload)

        path = f"/datasets/search/v1"
        response = self.apiobject.call_endpoint(path, payload, "post")
        return response
    
    def list_focus_configurations(self, token, client_id, oauth_secret, tenant_id, search_term = None):

        payload = {
            "token": token,
            "client_id": client_id,
            "oauth_secret": oauth_secret,
            "tenant_id": tenant_id,
            "search_term": search_term
        }

        validate_list_focus_configurations(payload)


        path = f"/datasets/focus-configurations/v1"
        response = self.apiobject.call_endpoint(path, payload, method="post")
        return response
    
    def get_focus_configuration(self, token, client_id, oauth_secret, tenant_id, focus_chat_id):

        payload = {
            "token": token,
            "client_id": client_id,
            "oauth_secret": oauth_secret,
            "tenant_id": tenant_id,
            "focus_chat_id": focus_chat_id
        }

        validate_get_focus_configuration(payload)

        path = f"/datasets/get-focus-configuration/v1"
        response = self.apiobject.call_endpoint(path, payload, method="post")
        return response
    
    def update_focus_configuration(self, token, client_id, oauth_secret, tenant_id, focus_chat_id, configuration):

        payload = {
            "token": token,
            "client_id": client_id,
            "oauth_secret": oauth_secret,
            "tenant_id": tenant_id,
            "focus_chat_id": focus_chat_id,
            "configuration": configuration
        }

        validate_update_focus_configuration(payload)

        path = f"/datasets/update-focus-configuration/v1"
        response = self.apiobject.call_endpoint(path, payload, method="post")
        return response


import os
import requests
import base64
from kbrainsdk.admin import Admin
from kbrainsdk.llms import LLMs
from kbrainsdk.datasets import Datasets
from kbrainsdk.ingest import Ingest
from kbrainsdk.ai import AI
from kbrainsdk.users import User
from kbrainsdk.events import Events
from kbrainsdk.signalr import SignalR

VALID_MODEL_NAMES = ["gpt-3.5-turbo", "gpt-4", "gpt-3.5-instruct"]

class KBRaiNAPI:
    def __init__(self, base_url=None, account_id=None, api_key=None):
        self.KBRAIN_NAME = "KBRaiN ™"
        self.base_url = base_url or os.getenv('KBRAIN_BASE_URL')
        account_id = account_id or os.getenv('KBRAIN_ACCOUNT_ID')
        api_key = api_key or os.getenv('KBRAIN_API_KEY')
        auth_string = f"{account_id}:{api_key}"
        auth_bytes = base64.b64encode(auth_string.encode('utf-8')).decode('utf-8')
        self.headers = {'Authorization': f'Basic {auth_bytes}'}
        self.admin = Admin(self)
        self.llms = LLMs(self)
        self.datasets = Datasets(self)
        self.ingest = Ingest(self)
        self.ai = AI(self)
        self.users = User(self)
        self.events = Events(self)
        self.signalr = SignalR(self)
        
    def healthy(self):
        path = f"/health/v1"
        response = self.call_endpoint(path, {}, "get")
        return response

    
    def call_endpoint(self, path, payload, method):
        request_method = getattr(requests, method)
        endpoint = f"{self.base_url}{path}"
        response = request_method(url=endpoint, json=payload, headers=self.headers)
        if response.status_code >= 200 and response.status_code <= 300:
            return response.json()
        
        message = response.json().get('message')
        raise Exception(message)